/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    buffer.h

    This is a header file for buffer management.

    2009-10-15	CF Lin
		new release
*/
#ifndef _BUFFER_H_
#define _BUFFER_H_
#include "os_dependent.h"

typedef unsigned long (*buffer_access_t) (void *dest, const void *src, unsigned long size);

#define ISBUFDATATHERE(b)	(b->datasize != b->consumed)
#define ISBUFAVAILABLE(b)	(b->datasize != b->datamax)

typedef struct _BUFFER
{
    char    *data;		/* allocated data */
    int     datasize;		/* total bytes used in `data' */
    int     datamax;		/* size of allocated memory block */
    int     consumed;		/* how many bytes of data consumed from this buffer */
    struct _BUFFER *next;
} BUFFER;

BUFFER*	buffer_new (int size);
void buffer_reset(BUFFER *b);
void buffer_free (BUFFER *b);
int buffer_queue (BUFFER *b, void *src, int max, buffer_access_t bufx);
int buffer_popup (BUFFER *b, void *src, int max, buffer_access_t bufx);
BUFFER* buffer_append (BUFFER *head, BUFFER *tail);
BUFFER* buffer_push (BUFFER *head, BUFFER *b);

#define buffer_popup_to_device  (b,p,x)		buffer_popup(b,p,0,x)
#define buffer_queue_from_device(b,p,x)		buffer_queue(b,p,0,x)

#endif

